;;;======================
;;; TONAL ANALYSIS IN OM
;;; Using Java objects
;;;======================

(in-package :ha)

; TODO: I need a function to be able to save patches using the library

;;;============
;;; CHORD INFO
;;;============

(defclass! chord-info ()
  ((root :accessor root :initarg :root :initform "C" :documentation "Chord root")
   (kind :accessor kind :initarg :kind :initform "" :documentation "Chord type")
   (weight :accessor weight :initarg :weight :initform nil :documentation "Chord weight")))

(defmethod get-chord-label ((self chord-info))
  (concatenate 'string (root self) (kind self)))

;;;=====================
;;; SEGMENT COORDINATES
;;;=====================
(defstruct segment-coordinates
  label-x
  label-y
  segment-top
  segment-bottom
  segment-left
  segment-right
  segment-width
  segment-height)

;;;==================
;;; HARMONIC SEGMENT
;;;==================

;;; importing slot accessor symbols from om::time-segments
(export '(om::t1 om::t2) :om)

;XXXXXXXXXXXXXXXXXXX; 
(defclass! harmonic-segment (om::time-segment)
  ((id :accessor id :initarg :id :initform 0 :documentation "Segment ID")
   (onset :accessor onset :initarg :onset :initform 0 :documentation "Onset time (ms)")
   (offset :accessor offset :initarg :offset :initform 0 :documentation "Offset time (ms)")
   (active :accessor active :initarg :active :initform nil :documentation "True if this is the current segment being validated by the user")
   (validated :accessor validated :initarg :validated :initform nil :documentation "True if the user has already validated this segment")
   (modified :accessor modified :initarg :modified :initform nil :documentation "True if the user has modified this segment")
   (previous :accessor previous :initarg :previous :initform nil :documentation "Previous segment")
   (next :accessor next :initarg :next :initform nil :documentation "Next segment")
   (parent :accessor parent :initform nil :documentation "Parent analysis")))

;XXXXXXXXXXXXXXXXXXX; 
(defclass! harmonic-data () 
  ((chord :accessor chord :initarg :chord :initform nil :documentation "Chord info")
   (chord-list :accessor chord-list :initarg :chord-list :initform nil :documentation "List of possible chords for this segment")))

(defmethod om::segment-begin ((self harmonic-segment))
  (onset self))

(defmethod om::segment-end ((self harmonic-segment))
  (offset self))

(defmethod get-segment-coordinates ((self harmonic-segment) view)
  (let* ((size (get-note-size view))
         (staff-top (get-staff-top-px view))
         (staff-h (get-staff-height-px view))
         (staff-line-h (get-staff-line-height-px view))
         (segment-top (- staff-top (* 8 staff-line-h)))
         (segment-bottom (+ (+ staff-top staff-h) (* 12 staff-line-h)))
         (segment-left (get-onset-in-px (onset self) view))
         (segment-right (- (get-onset-in-px (offset self) view) 2))
         ; If the previous segment overlaps, this segment is drawn a line down
         ;(prev (previous self))
         (line 0) ;(if prev (if (> (+ (get-onset-in-px (onset prev) view) (min-width prev)) xonset) 1 0) 0))
         (label-x (+ segment-left 8))
         (label-y (+ segment-bottom -4 (* line 20)));(- staff-top (* 6 staff-line-h) 4))
         ;(label-w (oa::om-string-size label font))
         ;(label-h (oa::om-string-h font))
         )
    (make-segment-coordinates :label-x label-x
                              :label-y label-y
                              :segment-top segment-top
                              :segment-bottom segment-bottom
                              :segment-left segment-left
                              :segment-right segment-right
                              :segment-width (- segment-right segment-left)
                              :segment-height (- segment-bottom segment-top))))

(defmethod om::draw-segment ((self harmonic-segment) view) 
  (let* ((font oa::*om-default-font2b*)
         (color (om::color self))
         (label (get-chord-label (chord (segment-data self)))) ;XXXXXXXXXXXXXXXXXXX; 
         (coord (get-segment-coordinates self view))
         (x (segment-coordinates-segment-left coord))
         (y1 (segment-coordinates-segment-top coord))
         (y2 (segment-coordinates-segment-bottom coord))
         (width (segment-coordinates-segment-width coord))
         (height (segment-coordinates-segment-height coord))
         (label-x (segment-coordinates-label-x coord))
         (label-y (segment-coordinates-label-y coord)))

    ; Draws the label
    (oa::om-with-font font                   
       (oa::om-with-fg-color view color
         (oa::om-draw-string label-x label-y label)))

    ; Draws the segment
    (oa::om-with-line-size 1
      (oa::om-with-line :dash
        (oa::om-with-fg-color view color
          (oa::om-draw-line x y1 x y2))
          (when (active self)
            ; (om-with-hilite-foreground
            (om-with-fg-color view (om-make-color-alpha 0 0 1 0.25)
            (om-fill-rect x y1 width height)))
          (when (validated self)
            ; (om-with-hilite-foreground
            (om-with-fg-color view (om-make-color-alpha 0 1 0 0.25)
              (om-fill-rect x y1 width height)))
))
    (when (om::selected self) (om-draw-hilite-rect x y1 width height))
    ;(call-next-method)
    ))

;;;=====================
;;; ANALYSIS VOCABULARY
;;;=====================

(defclass! ANALYSIS-VOCABULARY ()
  ((roots :accessor roots :initarg :roots :initform '(("C" 0) ("C#" 1) ("D" 2) ("D#" 3) ("E" 4) ("F" 5) ("F#" 6) ("G" 7) ("G#" 8) ("A" 9) ("A#" 10) ("B" 11)) :documentation "List in the form ('label' semitones-from-C)")
   (kinds :accessor kinds :initarg :kinds :initform '(("maj" (0 4 7)) ("m" (0 3 7)) ("dim" (0 3 6)) ("7" (0 4 7 10)) ("fdim" (0 3 6 9)) ("hdim" (0 3 6 10))) :documentation "List in the form ('label' (semitones from root))")))

(defmethod get-chord-profile ((self analysis-vocabulary) (chord chord-info))
  (let* ((root-data (find (root chord) (roots self) :key #'first :test #'equal))
         (root (if root-data (second root-data) 0))
         (kind-data (find (kind chord) (kinds self) :key #'first :test #'equal))
         (profile (if kind-data (second kind-data) '(0))))
    (mapcar #'(lambda (x) (+ x root)) profile)))
                    

;;;===================
;;; HARMONIC ANALYSIS
;;;===================


;;; warning: only arguments with :initarg are copied and saved with the instance
(defclass! HARMONIC-ANALYSIS (om::ABSTRACT-ANALYSIS)
  ((id :accessor id :initarg :id :initform nil :documentation "Analysis ID asigned by the server, for synchronization purposes only")
   (vocabulary :accessor vocabulary :initarg :vocabulary :initform (make-instance 'analysis-vocabulary) :documentation "Vocabulary used in the analysis")
   (segmentation :accessor segmentation :initarg :segmentation :initform nil :documentation "List of onset times (ms). When set, segmentation step is skipped and chord labels are computed for the segmentation defined by the list")
   (current-segment :accessor current-segment :initarg :current-segment :initform nil :documentation "Segment currently being validated by the user")
   (offsets :accessor offsets :initform nil :initarg :offsets :documentation "List of points in the score (in ms) where segments can end")))


;;; This analysis compute/analyse segments in a single step
(defmethod om::compute-segments-p ((self HARMONIC-ANALYSIS)) nil)
(defmethod om::analyse-segments-p ((self HARMONIC-ANALYSIS)) nil)
(defmethod om::compute+analyse-segments-p ((self HARMONIC-ANALYSIS)) t)


(defmethod om::segment-data-tostring ((self harmonic-analysis) (seg om::segment)) "")

;XXXXXXXXXXXXXXXXXXX; 
(defmethod om::segment-data-tostring ((self harmonic-analysis) (seg harmonic-segment))
  (get-chord-label (chord (segment-data seg))))

(defmethod om::compute-analysis-segments ((self harmonic-analysis) (object om::chord-seq))
  (let* ((path (om::save-as-midi object (tmpfile "in-file.mid")))
         (id (send-new-analysis self path)))
    (setf (id self) id)
    (if (and id (send-analysis-options self))
        (let ((result (send-execute-analysis self)))
          (if (and result (retrieve-offsets self))
              (let* ((segments (build-segments-from-textfile self result))
                     (segment (first segments)))
                (setf (active segment) t)
                (setf (current-segment self) segment)
                segments)
            (server-error self)))
      (server-error self))))

(defmethod om::compute-analysis-segments ((self harmonic-analysis) (object om::voice))
  (let* ((path (om::save-as-midi (om::objfromobjs object (make-instance 'om::chord-seq)) 
                                 (tmpfile "in-file.mid")))
         (id (send-new-analysis self path)))
    (setf (id self) id)
    (if (and id (send-analysis-options self))
        (let ((result (send-execute-analysis self)))
          (if (and result (retrieve-offsets self))
              (let* ((segments (build-segments-from-textfile self result))
                     (segment (first segments)))
                (setf (active segment) t)
                (setf (current-segment self) segment)
                segments)
            (server-error self)))
      (server-error self))))

(defmethod om::analysis-init ((self harmonic-analysis) object) 
  ;(om::object-analysis self object)
  (call-next-method)
  )

(defmethod server-error ((self harmonic-analysis))
  (om-beep-msg "Error in analysis !")
  nil)

(defmethod send-new-analysis ((self harmonic-analysis) (path string))
  (osc-send-receive "/analysis/harmonic" "new" path))

(defmethod send-analysis-options ((self harmonic-analysis))
  (send-segmentation self))

(defmethod send-set-option ((self harmonic-analysis) (name string) (value string))
  (osc-send-receive "/analysis/harmonic" "set" (id self) name value))

(defmethod send-segmentation ((self harmonic-analysis))
  (if (segmentation self)
      (let ((path (save-data (list (segmentation self)) (tmpfile "onsets.txt"))))
        (send-set-option self "input-onsets" (namestring path)))
    t))

(defmethod send-execute-analysis ((self harmonic-analysis))
  (osc-send-receive "/analysis/harmonic" "analyze" (id self)))

(defmethod retrieve-offsets ((self harmonic-analysis))
  (let ((response (osc-send-receive "/analysis/harmonic" "get-offsets" (id self))))
    (if response
        (build-offsets-from-textfile self response)
      nil)))

(defmethod send-segment-validated ((self harmonic-analysis) (id integer))
  (osc-send-receive "/analysis/harmonic" "validate" (id self) (write-to-string id)))

(defmethod replace-from-current-segment ((self harmonic-analysis) (path string))
  (let* ((segments (build-segments-from-textfile self path))
         (cur-id (id (current-segment self))))
    (setf (om::analysis-segments self)
          (x-append
           (delete-if #'(lambda (s) (>= (id s) cur-id)) (om::analysis-segments self))
           segments))
    (setf (active (first segments)) t)
    (setf (current-segment self) (first segments))))

(defmethod send-segment-offset ((self harmonic-analysis) (offset integer))
  (let ((segment (current-segment self)))
    (when segment
      (let ((response (osc-send-receive "/analysis/harmonic" "offset" (id self) (write-to-string (id segment)) (write-to-string offset))))
        (when response
          (replace-from-current-segment self response))))))

(defmethod send-segment-chord ((self harmonic-analysis) (id integer) (root string) (kind string))
  (let ((response (osc-send-receive "/analysis/harmonic" "label" (id self) (write-to-string id) root kind)))
    (when response
      (replace-from-current-segment self response))))


(defmethod build-segments-from-textfile ((self harmonic-analysis) (path string))
  (with-open-file (in path)
    (loop for line = (read-line in nil 'eof)
          until (eq line 'eof)
          collect
           ;XXXXXXXXXXXXXXXXXXXXXX;
          (let* ((filedata (om::data-from-line line))
                 (chord (nth (1+ (position :chord filedata)) filedata))
                 (clist (nth (1+ (position :chord-list filedata)) filedata))
                 (id (nth (1+ (position :id filedata)) filedata))
                 (ons (nth (1+ (position :onset filedata)) filedata))
                 (offs (nth (1+ (position :offset filedata)) filedata))
                 (seg (make-instance 'harmonic-segment 
                                     :active nil :validated nil 
                                     :id id :onset ons :offset offs
                                     :segment-data (make-instance 'harmonic-data
                                                                  :chord (apply #'make-instance 'chord-info chord)
                                                                  :chord-list (loop for c in clist collect
                                                                                    (apply #'make-instance 'chord-info c))))))
            ;XXXXXXXXXXXXXXXXXXXXXX;
            (setf (parent seg) self)
            seg))))

(defmethod build-offsets-from-textfile ((self harmonic-analysis) (path string))
  (with-open-file (in path)
    (let* ((line (read-line in nil 'eof)))
      (setf (offsets self) (om::data-from-line line))
      t)))

;(setf data (data-from-line ":id 10 :chord (:root 0 :kind \"maj\") :chord-list ((:root 0 :kind \"maj\")(:root 9 :kind \"m\"))"))
;(setf seg (apply #'make-instance 'ha::harmonic-segment data))
;(setf (ha::chord seg) (apply #'ha::make-chord-info (ha::chord seg)))
;(setf (ha::chord-list seg) (loop for chord-info in (ha::chord-list seg) collect (apply #'ha::make-chord-info chord-info)))

(defmethod validate-segment ((self harmonic-analysis) segment panel)
  (let* ((id (id segment))
         (response (send-segment-validated self id)))
    (when response
      (let ((nextId (car (om::data-from-line response))))
        (setf (current-segment self) nil)
        (loop for seg in (om::analysis-segments self)
              while (not (current-segment self))
              do
              (when (or (< (id seg) nextId) (= -1 nextId))
                (setf (validated seg) t)
                (setf (active seg) nil))
              (when (= (id seg) nextId)
                (setf (active seg) t)
                (setf (current-segment self) seg)
                )))
      (om::update-panel panel))))

(defmethod change-segment-chord ((self harmonic-analysis) segment panel &key on-segment)
  (when (active segment)
    (let ((options (list
                    (loop for chord in (chord-list (segment-data segment)) ;XXXXXXXXXXXXXXXXXXXXXX;
                          collect
                          (let ((r (root chord))
                                (k (kind chord)))
                            (om-new-leafmenu (get-chord-label chord)
                                             #'(lambda () (send-segment-chord self (id segment) r k) (om::update-panel panel)))))
                   (loop for root in (roots (vocabulary self))
                         collect
                         (apply #'om-new-menu
                                (first root)
                                (loop for kind in (kinds (vocabulary self))
                                      collect
                                      (let* ((r (first root))
                                             (k (first kind))
                                             (chord (concatenate 'string r k)))
                                        (om-new-leafmenu chord
                                                         #'(lambda () (send-segment-chord self (id segment) r k) (om::update-panel panel))))))))))
      (if on-segment
          (let* ((coord (get-segment-coordinates segment panel))
                 (x (+ 10 (segment-coordinates-label-x coord)))
                 (y (+ 10 (segment-coordinates-label-y coord))))
            (capi:display-popup-menu (funcall #'om-make-menu "Change chord label" options) :owner panel :x x :y y))
        (om-open-pop-up-menu (funcall #'om-make-menu "Change chord label" options) panel))
      )))

(defmethod reduce-segment-offset ((self harmonic-analysis))
  (when (current-segment self)
    (let* ((segment (current-segment self))
           (offsets (offsets self))
           (offset (find (offset segment) offsets :test '> :from-end t)))
      (when (not (<= offset (onset segment)))
        (send-segment-offset self offset)))))

(defmethod expand-segment-offset ((self harmonic-analysis))
  (when (current-segment self)
    (let* ((segment (current-segment self))
           (offsets (offsets self))
           (offset (find (offset segment) offsets :test '<)))
      (when offset
        (send-segment-offset self offset)))))

(defmethod play-segment ((self harmonic-analysis))
  (when (current-segment self)
    (let* ((chord (chord (segment-data (current-segment self)))) ;XXXXXXXXXXXXXXXXXXXXXX;
           (profile (get-chord-profile (vocabulary self) chord)))
      (play (make-instance 'chord :lmidic (mapcar #'(lambda (x) (* 100 (+ 60 x))) profile)))
      )))

(defmethod play-segment-score ((self harmonic-analysis) panel)
  (when (current-segment self)
    (let* ((segment (current-segment self))
           (interval (list (onset segment) (- (offset segment) 1))))
      (setf (om::cursor-interval panel) interval)
      (om::play-selection-from-palette panel)
      (setf (om::cursor-interval panel) '(0 0)))))

(defmethod om::analysis-key-event ((self harmonic-analysis) panel char)
  (case char 
    (:om-key-delete (om::om-beep))
    (:om-key-left (when (om-shift-key-p) (reduce-segment-offset self) (om::update-panel panel)))
    (:om-key-right (when (om-shift-key-p) (expand-segment-offset self) (om::update-panel panel)))
    (#\h (show-help-window))
    (#\p (play-segment self))
    (#\P (play-segment-score self panel))
    (#\v (when (current-segment self) (validate-segment self (current-segment self) panel)))
    (#\l (when (current-segment self) (change-segment-chord self (current-segment self) panel :on-segment t)))
    (otherwise (call-next-method))))

;;; !!! solo chord-seq !!!
(defmethod get-onset-list ((obj chord-seq))
  (nconc (loop for chord in (om::inside obj)
               collect (om::offset->ms chord))
         (list (om::extent->ms obj))))


(defmethod om::analysis-add-click ((self harmonic-analysis) panel pos)
  (when (current-segment self)
    (let* ((offsets (offsets self));(get-onset-list (om::object (om::editor panel))))
           (move-selection #'(lambda (view pos)
                               (let ((rect (om-get-rect-movable-object panel (om-point-h pos) (om-point-v pos))))
                                 (when rect
                                   (let* ((x (om::pixel-toms panel (om-make-point (+ (first rect) (third rect)) 0)))
                                          (x-onset (max (or (find x offsets :test '<) (car (last offsets))) 
                                                        (nth (+ 1 (position (onset (current-segment self)) offsets :test '=)) offsets))
                                                   ))
                                     (om-update-movable-object panel (get-onset-in-px (onset (current-segment self)) panel) 0 
                                                               (- (get-onset-in-px x-onset panel) 
                                                                  (get-onset-in-px (onset (current-segment self)) panel))
                                                               (om-point-v (om-interior-size panel))))))
                               ))
           (release-selection #'(lambda (view pos)
                                  (om::release-interval-select view pos)
                                  (let* ((x (om::pixel-toms panel pos))
                                         (x-onset (or (find x offsets :test '<) (car (last offsets))))
                                         (offset (max x-onset 
                                                      (nth (+ 1 (position (onset (current-segment self)) offsets :test '=)) offsets))))
                                    (send-segment-offset self offset)
                                    (om::update-panel panel)
                                    ))))
    
      (om-init-motion-functions panel move-selection release-selection)
      (om-new-movable-object panel (get-onset-in-px (onset (current-segment self)) panel) 0 
                             (- (om-point-h pos) (get-onset-in-px (onset (current-segment self)) panel))
                             (om::h panel) 'om-selection-rectangle)
      self)
  ))

(defmethod om::handle-segment-doubleclick ((self harmonic-analysis) segment panel pos)
  (when (not (validated segment))
    (if (om-shift-key-p)
        ; SHIFT + DOUBLE CLICK: Change chord label
        (change-segment-chord self segment panel)
    ; DOUBLE CLICK: Validate segment
    (validate-segment self segment panel)
    )))

  ;(om-make-window 'om-window)
;  (if (om-shift-key-p)
;      (let ((str (om-get-user-string "Enter new label" :initial-string (om::segment-data-tostring self segment))))
;        (when str (print str))))
;    (om-open-pop-up-menu (om-new-menu "ACTIONS" 
;                                      (om-new-leafmenu  "AAA" ;;; name
;                                                        #'(lambda () (print (list segment "A")))  ;;; action
;                                                        nil ;;; shortcut (window menu only) -- optional
;                                                        nil ;;; enabled (T or NIL) -- optional 
;                                                        )
;                                      (om-new-leafmenu "BBB"
;                                                       #'(lambda () (print (list segment "B")))))
;                         panel)
;    )
                                                                 

;;; ADDITIONAL ITEMS EN EL MENU CONTEXTUAL (RIGHT-CLICK)
;(defmethod om::get-analysis-menu-items ((self harmonic-analysis) &optional editor)
  ;(list (om-new-leafmenu "Action 1"
  ;                       #'(lambda () (print "1")))
  ;      (om-new-leafmenu "Action 2"
  ;                       #'(lambda () (print "2")))
  ;      )
;  nil)


(defmethod om::omNG-save ((self harmonic-analysis) &optional (values? nil))
  `(let ((analysis ,(call-next-method)))
     ;;; (llama servidor)
     analysis))



(defun show-help-window ()
  (om::show-help-window "Commands for Harmonic analysis" 
                        '(((alt+clic "New Object")
                           (del "Delete Selection")
                           (("c") "Show Channel Color")
                           (("C") "Change Selection Color")
                           (("o") "Open Selection Internal Editor")
                           (ud "Transpose Selection")
                           (lr "Change Selection Offset/Duration")
                           (space "Play/Stop"))
                          ((alt+clic "New Object")
                           (del "Delete Selection")
                           (("c") "Show Channel Color")
                           (("C") "Change Selection Color")
                           (("o") "Open Selection Internal Editor")
                           (ud "Transpose Selection")
                           (lr "Change Selection Offset/Duration")
                           (space "Play/Stop")))))

;;;===============
;;; TREE ANALYSIS
;;;===============


(defclass! tree-segment (om::segment)
  ((onset :accessor onset :initarg :onset :initform nil :documentation "")))

(defmethod om::draw-segment ((self tree-segment) view)
  (let* ((children (remove-if #'null (loop for seg in (om::segment-data self)
                                           collect
                                           (om::draw-segment seg view))))
         (x 0)
         (y 0)
         (staff-bottom (+ 25 (get-staff-top-px view )(get-staff-height-px view))))
    (print children)
    (cond
     ((= (length children) 0)
      (setf x (onset self))
      (setf y 1))
     ((= (length children) 1)
      (setf x (caar children))
      (setf y (+ 1 (cadar children))))
     ((> (length children) 1)
      (setf x (+ (caar children) (round (/ (- (caar (last children)) (caar children)) 2))))
      (setf y (+ 1 (apply #'max (mapcar #'second children))))))
    (loop for child in children
          do
          (om::om-draw-line (+ 2 (get-onset-in-px x view)) (+ 2 staff-bottom (* y 40)) (+ 2 (get-onset-in-px (car child) view)) (+ 2 staff-bottom (* (cadr child) 40))))
    (om::om-draw-rect (get-onset-in-px x view) (+ staff-bottom (* y 40)) 4 4)
    (print (list x y))))

(defmethod om::segment-begin ((self tree-segment))
  (onset self))

(defmethod om::segment-end ((self tree-segment))
  (onset self))

(defclass! TREE-ANALYSIS (om::ABSTRACT-ANALYSIS)
  ())

;;; This analysis compute/analyse segments in a single step
(defmethod om::compute-segments-p ((self tree-analysis)) nil)
(defmethod om::analyse-segments-p ((self tree-analysis)) nil)
(defmethod om::compute+analyse-segments-p ((self tree-analysis)) t)

;;; This analysis only works with voices
(defmethod om::compatible-analysis-p ((analyse tree-analysis) (object om::voice)) t)
(defmethod om::compatible-analysis-p ((analyse tree-analysis) (object t)) nil)

(defmethod compute-tree-segments ((self tree-analysis) (obj t) (parent-onset integer))
  nil)

(defmethod compute-segments-recursive ((self tree-analysis) (obj t) (parent-onset integer))
  (let* ((onset (+ parent-onset (om::offset->ms obj)))
         (segment (make-instance 'tree-segment :onset onset)))
    (setf (om::segment-data segment)
          (remove-if #'null
                     (loop for x in (om::inside obj)
                           collect
                           (compute-tree-segments self x onset))))
    segment))

(defmethod compute-tree-segments ((self tree-analysis) (obj om::measure) (parent-onset integer))
  (compute-segments-recursive self obj parent-onset))

(defmethod compute-tree-segments ((self tree-analysis) (obj om::group) (parent-onset integer))
  (compute-segments-recursive self obj parent-onset))

(defmethod compute-tree-segments ((self tree-analysis) (obj om::chord) (parent-onset integer))
  (compute-segments-recursive self obj parent-onset))

(defmethod om::compute-analysis-segments ((self tree-analysis) (object om::voice))
  (let ((segment (make-instance 'tree-segment :onset 0)))
    (setf (om::segment-data segment)
          (loop for measure in (om::inside object)
                collect
                (compute-tree-segments self measure 0)))
    (list! segment)))


;;;======================
;;; GROUNDTRUTH ANALYSIS
;;;======================

(defclass! groundtruth-segment (om::segment)
  ((onset :accessor onset :initarg :onset :initform 0)))

(defmethod om::draw-segment ((self groundtruth-segment) view) 
  (let* ((p1 (om::time-to-pixels view (onset self))))
    (om-with-line-size (if (om::selected self) 3 1.5)
      (om-with-line :solid
      (om-with-fg-color view (om::color self)
        (om-draw-line p1 0 p1 (om::h view))
        (oa::om-with-font oa::*om-default-font2b* 
                          (oa::om-draw-string (+ p1 10) 20 (om::segment-data self))))
        ))))

(defclass! GROUNDTRUTH-ANALYSIS (om::ABSTRACT-ANALYSIS)
  ((path :accessor path :initarg :path :initform nil)))

;;; This analysis compute/analyse segments in a single step
(defmethod om::compute-segments-p ((self GROUNDTRUTH-ANALYSIS)) nil)
(defmethod om::analyse-segments-p ((self GROUNDTRUTH-ANALYSIS)) nil)
(defmethod om::compute+analyse-segments-p ((self GROUNDTRUTH-ANALYSIS)) t)


(defun split-by-one-space (string)
    "FROM: http://cl-cookbook.sourceforge.net/strings.html
Returns a list of substrings of string
divided by ONE space each.
Note: Two consecutive spaces will be seen as
if there were an empty string between them."
    (loop for i = 0 then (1+ j)
          as j = (position #\Space string :start i)
          collect (subseq string i j)
          while j))

(defmethod om::compute-analysis-segments ((self groundtruth-analysis) (object om::chord-seq))
  (when (not (path self))
    (setf (path self) (file-chooser)))
  (with-open-file (in (path self))
    (loop for line = (read-line in nil 'eof)
          until (eq line 'eof)
          collect
          (let* ((data (split-by-one-space line))
                 (chord (first data))
                 (time (read-from-string (second data))))
            (print chord)
            (print time)
            (make-instance 'groundtruth-segment :onset time :segment-data chord :color om::*om-red2-color*)))))
